"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.HighlightJsLineNumbers = void 0;
var HighlightJsLineNumbers = /** @class */ (function () {
    function HighlightJsLineNumbers() {
    }
    HighlightJsLineNumbers.isHljsLnCodeDescendant = function (domElt) {
        var curElt = domElt;
        while (curElt) {
            if (curElt.className && curElt.className.indexOf('hljs-ln-code') !== -1) {
                return true;
            }
            curElt = curElt.parentNode;
        }
        return false;
    };
    HighlightJsLineNumbers.getHljsLnTable = function (hljsLnDomElt) {
        var curElt = hljsLnDomElt;
        while (curElt.nodeName !== 'TABLE') {
            curElt = curElt.parentNode;
        }
        return curElt;
    };
    HighlightJsLineNumbers.edgeGetSelectedCodeLines = function (selection) {
        // current selected text without line breaks
        var selectionText = selection.toString();
        // get the <td> element wrapping the first line of selected code
        var tdAnchor = selection.anchorNode;
        while (tdAnchor.nodeName !== 'TD') {
            tdAnchor = tdAnchor.parentNode;
        }
        // get the <td> element wrapping the last line of selected code
        var tdFocus = selection.focusNode;
        while (tdFocus.nodeName !== 'TD') {
            tdFocus = tdFocus.parentNode;
        }
        // extract line numbers
        var firstLineNumber = parseInt(tdAnchor.dataset.lineNumber);
        var lastLineNumber = parseInt(tdFocus.dataset.lineNumber);
        // multi-lines copied case
        if (firstLineNumber != lastLineNumber) {
            var firstLineText = tdAnchor.textContent;
            var lastLineText = tdFocus.textContent;
            // if the selection was made backward, swap values
            if (firstLineNumber > lastLineNumber) {
                var tmp = firstLineNumber;
                firstLineNumber = lastLineNumber;
                lastLineNumber = tmp;
                tmp = firstLineText;
                firstLineText = lastLineText;
                lastLineText = tmp;
            }
            // discard not copied characters in first line
            while (selectionText.indexOf(firstLineText) !== 0) {
                firstLineText = firstLineText.slice(1);
            }
            // discard not copied characters in last line
            while (selectionText.lastIndexOf(lastLineText) === -1) {
                lastLineText = lastLineText.slice(0, -1);
            }
            // reconstruct and return the real copied text
            var selectedText = firstLineText;
            var hljsLnTable = this.getHljsLnTable(tdAnchor);
            for (var i = firstLineNumber + 1; i < lastLineNumber; ++i) {
                var codeLineSel = this.format('.{0}[{1}="{2}"]', [
                    this.CODE_BLOCK_NAME,
                    this.DATA_ATTR_NAME,
                    i,
                ]);
                var codeLineElt = hljsLnTable.querySelector(codeLineSel);
                selectedText += '\n' + codeLineElt.textContent;
            }
            selectedText += '\n' + lastLineText;
            return selectedText;
            // single copied line case
        }
        else {
            return selectionText;
        }
    };
    HighlightJsLineNumbers.addStyles = function (document) {
        var css = document.createElement('style');
        css.type = 'text/css';
        css.innerHTML = this.format('.{0}{border-collapse:collapse}' +
            '.{0} td{padding:0}' +
            '.{1}:before{content:attr({2})}', [this.TABLE_NAME, this.NUMBER_LINE_NAME, this.DATA_ATTR_NAME]);
        document.getElementsByTagName('head')[0].appendChild(css);
    };
    HighlightJsLineNumbers.isPluginDisabledForBlock = function (element) {
        return element.classList.contains('nohljsln');
    };
    HighlightJsLineNumbers.lineNumbersBlock = function (element, options) {
        var _this = this;
        if (typeof element !== 'object')
            return;
        this.async(function () {
            element.innerHTML = _this.lineNumbersInternal(element, options);
        });
    };
    HighlightJsLineNumbers.lineNumbersValue = function (document, value, options) {
        if (typeof value !== 'string')
            return;
        var element = document.createElement('code');
        element.innerHTML = value;
        return this.lineNumbersInternal(element, options);
    };
    HighlightJsLineNumbers.lineNumbersInternal = function (element, options) {
        var internalOptions = this.mapOptions(element, options);
        this.duplicateMultilineNodes(element);
        return this.addLineNumbersBlockFor(element.innerHTML, internalOptions);
    };
    HighlightJsLineNumbers.addLineNumbersBlockFor = function (inputHtml, options) {
        var lines = this.getLines(inputHtml);
        // if last line contains only carriage return remove it
        if (lines[lines.length - 1].trim() === '') {
            lines.pop();
        }
        if (lines.length > 1 || options.singleLine) {
            var html = '';
            for (var i = 0, l = lines.length; i < l; i++) {
                html += this.format('<tr>' +
                    '<td class="{0} {1}" {3}="{5}">' +
                    '<div class="{2}" {3}="{5}"></div>' +
                    '</td>' +
                    '<td class="{0} {4}" {3}="{5}">' +
                    '{6}' +
                    '</td>' +
                    '</tr>', [
                    this.LINE_NAME,
                    this.NUMBERS_BLOCK_NAME,
                    this.NUMBER_LINE_NAME,
                    this.DATA_ATTR_NAME,
                    this.CODE_BLOCK_NAME,
                    i + options.startFrom,
                    lines[i].length > 0 ? lines[i] : ' ',
                ]);
            }
            return this.format('<table class="{0}">{1}</table>', [
                this.TABLE_NAME,
                html,
            ]);
        }
        return inputHtml;
    };
    /**
     * @param {HTMLElement} element Code block.
     * @param {Object} options External API options.
     * @returns {Object} Internal API options.
     */
    HighlightJsLineNumbers.mapOptions = function (element, options) {
        options = options || {};
        return {
            singleLine: this.getSingleLineOption(options),
            startFrom: this.getStartFromOption(element, options),
        };
    };
    HighlightJsLineNumbers.getSingleLineOption = function (options) {
        var defaultValue = false;
        if (!!options.singleLine) {
            return options.singleLine;
        }
        return defaultValue;
    };
    HighlightJsLineNumbers.getStartFromOption = function (element, options) {
        var defaultValue = 1;
        var startFrom = defaultValue;
        if (isFinite(options.startFrom)) {
            startFrom = options.startFrom;
        }
        // can be overridden because local option is priority
        var value = this.getAttribute(element, 'data-ln-start-from');
        if (value !== null) {
            startFrom = this.toNumber(value, defaultValue);
        }
        return startFrom;
    };
    /**
     * Recursive method for fix multi-line elements implementation in highlight.js
     * Doing deep passage on child nodes.
     * @param {HTMLElement} element
     */
    HighlightJsLineNumbers.duplicateMultilineNodes = function (element) {
        var nodes = element.childNodes;
        for (var node in nodes) {
            if (nodes.hasOwnProperty(node)) {
                var child = nodes[node];
                if (this.getLinesCount(child.textContent) > 0) {
                    if (child.childNodes.length > 0) {
                        this.duplicateMultilineNodes(child);
                    }
                    else {
                        this.duplicateMultilineNode(child.parentNode);
                    }
                }
            }
        }
    };
    /**
     * Method for fix multi-line elements implementation in highlight.js
     * @param {HTMLElement} element
     */
    HighlightJsLineNumbers.duplicateMultilineNode = function (element) {
        var className = element.className;
        if (!/hljs-/.test(className))
            return;
        var lines = this.getLines(element.innerHTML);
        var result = '';
        for (var i = 0; i < lines.length; i++) {
            var lineText = lines[i].length > 0 ? lines[i] : ' ';
            result += this.format('<span class="{0}">{1}</span>\n', [
                className,
                lineText,
            ]);
        }
        element.innerHTML = result.trim();
    };
    HighlightJsLineNumbers.getLines = function (text) {
        if (text.length === 0)
            return [];
        return text.split(this.BREAK_LINE_REGEXP);
    };
    HighlightJsLineNumbers.getLinesCount = function (text) {
        return (text.trim().match(this.BREAK_LINE_REGEXP) || []).length;
    };
    ///
    /// HELPERS
    ///
    HighlightJsLineNumbers.async = function (func) {
        setTimeout(func, 0);
    };
    /**
     * {@link https://wcoder.github.io/notes/string-format-for-string-formating-in-javascript}
     * @param {string} format
     * @param {array} args
     */
    HighlightJsLineNumbers.format = function (format, args) {
        return format.replace(/\{(\d+)\}/g, function (m, n) {
            return args[n] !== undefined ? args[n] : m;
        });
    };
    /**
     * @param {HTMLElement} element Code block.
     * @param {String} attrName Attribute name.
     * @returns {String} Attribute value or empty.
     */
    HighlightJsLineNumbers.getAttribute = function (element, attrName) {
        return element.hasAttribute(attrName)
            ? element.getAttribute(attrName)
            : null;
    };
    /**
     * @param {String} str Source string.
     * @param {Number} fallback Fallback value.
     * @returns Parsed number or fallback value.
     */
    HighlightJsLineNumbers.toNumber = function (str, fallback) {
        if (!str)
            return fallback;
        var number = Number(str);
        return isFinite(number) ? number : fallback;
    };
    HighlightJsLineNumbers.TABLE_NAME = 'hljs-ln';
    HighlightJsLineNumbers.LINE_NAME = 'hljs-ln-line';
    HighlightJsLineNumbers.CODE_BLOCK_NAME = 'hljs-ln-code';
    HighlightJsLineNumbers.NUMBERS_BLOCK_NAME = 'hljs-ln-numbers';
    HighlightJsLineNumbers.NUMBER_LINE_NAME = 'hljs-ln-n';
    HighlightJsLineNumbers.DATA_ATTR_NAME = 'data-line-number';
    HighlightJsLineNumbers.BREAK_LINE_REGEXP = /\r\n|\r|\n/g;
    return HighlightJsLineNumbers;
}());
exports.HighlightJsLineNumbers = HighlightJsLineNumbers;
